#include "qthing.h"
#include "io.h"
#include "environment.h"
#include "device/device_config.h"
#include "event.h"
#include "mqtt.h"
#include "network/udp.h"

#include "string.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include <lwip/sockets.h>
#include "esp_wifi.h"
#include "esp_log.h"
#include "esp_event_loop.h"
#include "nvs_flash.h"

#define TAG "network"

bool network_initialized = false;
bool clients_started = false;

void start_network_clients_task(void* arg) {
    start_udp_server();
    mqtt_client_start();
    vTaskDelete(NULL);
}

void start_network_clients() {
    if (!clients_started) {
        clients_started = true;
        xTaskCreate(start_network_clients_task, "start_network_clients_task", 4096, NULL, 7, NULL);
        log_oled("Network connection aquired");
    }
}

static esp_err_t event_handler(void *ctx, system_event_t *event)
{
    switch(event->event_id) {
    case SYSTEM_EVENT_STA_START:
        ESP_LOGI(TAG, "SYSTEM_EVENT_STA_START");
        update_wlan_connection_status(connecting);
        ESP_ERROR_CHECK(tcpip_adapter_set_hostname(TCPIP_ADAPTER_IF_STA, DEVICE_NAME));
        ESP_ERROR_CHECK(esp_wifi_connect());
        break;
    // case SYSTEM_EVENT_STA_CONNECTED:
    //     tcpip_adapter_create_ip6_linklocal(TCPIP_ADAPTER_IF_STA);
    //     break;
    case SYSTEM_EVENT_STA_GOT_IP:
        ESP_LOGI(TAG, "SYSTEM_EVENT_STA_GOT_IP");
        ESP_LOGI(TAG, "got ip:%s\n", ip4addr_ntoa(&event->event_info.got_ip.ip_info.ip));
        update_wlan_connection_status(connected);
        start_network_clients();
        break;
    case SYSTEM_EVENT_STA_DISCONNECTED:
        ESP_LOGI(TAG, "SYSTEM_EVENT_STA_DISCONNECTED");
        update_wlan_connection_status(disconnected);
        ESP_ERROR_CHECK(esp_wifi_connect());
        break;
    case SYSTEM_EVENT_AP_STA_GOT_IP6:
        ESP_LOGI(TAG, "SYSTEM_EVENT_AP_STA_GOT_IP6");
        ESP_LOGI(TAG, "IPv6: %s", ip6addr_ntoa(&event->event_info.got_ip6.ip6_info.ip));
        break;
    case SYSTEM_EVENT_ETH_START:
        ESP_LOGI(TAG, "SYSTEM_EVENT_ETH_START");
        update_eth_connection_status(disconnected);
        ESP_ERROR_CHECK(tcpip_adapter_set_hostname(TCPIP_ADAPTER_IF_ETH, DEVICE_NAME));
        break;
    case SYSTEM_EVENT_ETH_CONNECTED:
        ESP_LOGI(TAG, "SYSTEM_EVENT_ETH_CONNECTED");
        update_eth_connection_status(connecting);
        break;
    case SYSTEM_EVENT_ETH_DISCONNECTED:
        ESP_LOGI(TAG, "SYSTEM_EVENT_ETH_DISCONNECTED");
        update_eth_connection_status(disconnected);
        break;
    case SYSTEM_EVENT_ETH_GOT_IP:
        ESP_LOGI(TAG, "SYSTEM_EVENT_ETH_GOT_IP");
        ESP_LOGI(TAG, "got ip:%s\n", ip4addr_ntoa(&event->event_info.got_ip.ip_info.ip));
        update_eth_connection_status(connected);
        start_network_clients();
        break;
    default:
        break;
    }
    return ESP_OK;
}

void initialize_network() {
    if (network_initialized) {
        return;
    }
    network_initialized = true;

    ESP_LOGI(TAG, "initializing tcpip adapter");
    tcpip_adapter_init();

    ESP_LOGI(TAG, "starting network event loop");
    ESP_ERROR_CHECK(esp_event_loop_init(event_handler, NULL));
}
